#!/usr/bin/env python
#
#    LevelHold.py -- a pygtk Level/Hold widget
#
#    Copyright (C) 1999  Eric S. Tiedemann
#
#    This library is free software; you can redistribute it and/or
#    modify it under the terms of the GNU Library General Public
#    License as published by the Free Software Foundation; either
#    version 2 of the License, or (at your option) any later version.
#
#    This library is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#    Library General Public License for more details.
#
#    You should have received a copy of the GNU Library General Public
#    License along with this library; if not, write to the Free
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#    Contact: Eric S. Tiedemann, est@hyperreal.org
#

# move to C
#
# GtkObject *gtk_level_hold_new(int h, int w, int segments, int intensities)
# gtk_level_hold_set_level(double l)
# gtk_level_hold_fade()
#
# hmm..mebbe the monocolor case can be handled by gcs
#
# complete class rep in python b4 moving to C
#
# add configuration args
		
from gtk import *

MAXC = 0xFFFF

def hsb2rgb(h, s, v):
	#print h, s, v

	if h == None:
		return (int(MAXC * v),) * 3
	
	h = h * 6

	i = int(h)
	f = h - i

	if not i & 1: f = 1 - f

	m = v * (1 - s)
	n = v * (1 - s * f)

	if (i == 0 or i == 6):
		t = v, n, m
	elif (i == 1): t = n, v, m
	elif (i == 2): t = m, v, n
	elif (i == 3): t = m, n, v
	elif (i == 4): t = n, m, v
	elif (i == 5): t = v, m, n

	r, g, b = t[0] * MAXC, t[1] * MAXC, t[2] * MAXC
	
	#print r, g, b
	return r, g, b

def calccolors(cm, n):
	colors = []
	for x in range(0, n):
		colors = colors + [apply(cm.alloc,
					 (hsb2rgb(0, 1, float(x)/n)))]
	return colors

class GtkLevelHold(GtkDrawingArea):
	def configure_event(self, w, event):
		#print 'config', w, event
		win = self.get_window()
		self.pixmap = create_pixmap(win, win.width, win.height, -1)
		#self.set_style(self.get_style().copy())
		#print self.new_gc()
		draw_rectangle(self.pixmap, self.get_style().black_gc, TRUE,
			       0, 0, win.width, win.height)
		#print win.width, win.height
		self.normal = self.get_style().fg_gc[STATE_NORMAL]
		self.style = self.get_style()
		cm = self.get_window().colormap
		self.colors = calccolors(cm, self.intensities)
		
		return TRUE

	def expose_event(self, w, event):
		area = event.area
		self.draw_pixmap(self.normal, self.pixmap,
				 area[0], area[1], area[0],
				 area[1], area[2], area[3])
		return FALSE

	def set_level(self, l):
		if l > 1: l = 1.0
		if l < 0: l = 0.0
		i = int(l * self.segments)
		if (i <= self.last):
			self.last = i
			return

		for x in range(self.last, i+1):
			self.lv[x-1] = 1.0
		
		gc = self.style.mid_gc[0]#fg_gc[0]
		self.last = i
		self.maxnonzero = max(i, self.maxnonzero)
		gc.foreground = self.colors[-1]
		draw_rectangle(self.pixmap, gc, TRUE,
			       0, self.segsize * (self.segments - i),
			       self.w, i * self.segsize)
		self.queue_draw()
	
	def fade(self):
		# fade everything above last
		gc = self.style.mid_gc[0]#fg_gc[0]
		mnz = self.last
		#print self.last + 1, self.maxnonzero+1
		if (self.last < self.segments):
			#for x in range(self.last + 1, self.segments+1):
			for x in range(self.last + 1, self.maxnonzero+1):
				
		                #print x, lv[x-1]
				if self.lv[x-1] > 0:
					mnz = x
					if self.lv[x-1] < self.fuzz: self.lv[x-1] = 0.0
					else: self.lv[x-1] = self.lv[x-1] / 1.1
			                #print "fading %d to %f" % (x, self.lv[x-1])
					gc.foreground = self.colors[int(self.lv[x-1] * (self.intensities - 1))]
					draw_rectangle(self.pixmap, gc, TRUE, 0,
						       self.segsize * (self.segments - x),
						       self.w, self.segsize)
			self.maxnonzero = mnz
			self.queue_draw()
	
		return 1

	def __init__(self, h, w, segments, intensities):
		GtkDrawingArea.__init__(self)
		self.h = h
		self.w = w
		self.segments = segments
		self.intensities = intensities
		self.segsize = h / segments
		
		self.pixmap = None
		self.size(w, h)
		self.connect("expose_event", self.expose_event)
		self.connect("configure_event", self.configure_event)
		self.set_events(GDK.EXPOSURE_MASK | GDK.LEAVE_NOTIFY_MASK)

		self.colors = []
		self.last = 0
		self.lv = segments * [0.0]
		self.maxnonzero = 0
		self.factor = 1.1
		self.fuzz = .01
		self.style = None
		self.normal = None

if (__name__ == '__main__'):
	import sys
	import midiin
	import os
	import fcntl
	import FCNTL
	import whrandom

	global widget, colors, cl, cm, lh
	win = GtkWindow()
	win.set_name("Test Input")
	win.connect("destroy", mainquit)
	win.set_border_width(5)
	win.set_usize(50, 250)
	vbox = GtkVBox(spacing=3)
	win.add(vbox)
	vbox.show()

	a = GtkAlignment(.5, .5, 0.0, 0.0)
	a.show()
	vbox.pack_start(a)
	lh = GtkLevelHold(200, 10, 10, 20)
	lh.show()
	a.add(lh)
	
	button = GtkButton("Quit")
	vbox.pack_start(button, expand=FALSE, fill=FALSE)
	button.connect("clicked", win.destroy)
	button.show()
	win.show()

	timeout_add(200, lh.fade)

	def doit():
		lh.set_level(whrandom.random())
		return 1

        #timeout_add(50, doit)

	midi = os.open('/dev/midi', os.O_RDONLY)

	def cb(*arg):
		# print arg
		if (arg[0] == 'control-change' and arg[1] == 5 and arg[2] == 1):
			# print arg[3]
			lh.set_level(arg[3]/127.0)

	midiin.set_callback(cb)

	def handle_midi(fd, huh):
		bs = os.read(fd, 100)
		# sys.stderr.write("in handle_midi(%d)\n" % len(bs))
		midiin.inbytes(bs)

	fcntl.fcntl(midi, FCNTL.F_SETFL,
		    (os.O_NONBLOCK | fcntl.fcntl(midi, FCNTL.F_GETFL)))
	input_add(midi, GDK.INPUT_READ, handle_midi)


	if (0):
		import profile
		import pstats
		profile.run('mainloop()', 'levelstats')
		p = pstats.Stats('levelstats')
		p.sort_stats('cumulative').print_stats(10)
		p.sort_stats('time').print_stats(10)
	else:
		mainloop()
