#!/usr/bin/python
#                  -*- Python -*-
#
#    oolaboola -- user interface for digital dj program
#    Copyright (C) 1999  Eric S. Tiedemann
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

version = "0.3.0"

import os
import stat
import sys
import string
import sexpr
import MidiIn2
import posixpath
import code
# to make dorepl() nicer
import readline
import time
import fcntl
import FCNTL
from float2hms import *
import LevelHold
import math
import getopt
import oolalow

from gtk import *
import GDK

profilep = 0
nologp = 0
cue_channel = None

spacing = 2
border_width = 2

oolalog = None

last_time = time.time()

def doquit(*args):
  if (oolalog):
    try:
      oolalog.close()
    except IOError, e:
      print "problem closing `%s': %s\n" % (oolalogn, e[1])

  mainquit()
  
def out2mixd(s, nologp=0):
  global oolalog, last_time, mixdout
  if not mixdout: return
  #sys.stderr.write("out2mixd(%s)\n" % s)
  if oolalog and not nologp:
    nt = time.time()
    try: oolalog.write(("(delta %f " % (nt - last_time)) + s + ")\n")
    except IOError, e: print "problem writing to `%s': %s" % (oologn, e[1])
    last_time = nt
  try:
    mixdout.write(`s` + "\n")
    mixdout.flush()
  except IOError, e:
    print "problem talking to mixd: %s" % e[1]
    print "quitting..."
    # so we don't we don't end up here again
    mixdout = None
    doquit()
  
players = ()

def escape_string(s):
    s1 = "\""
    for c in s:
        if (c == "\"" or c == "\\"):
          s1 = s1 + "\\" + c
        elif c == "\n":
          s1 = s1 + "\\n"
        else:
            s1 = s1 + c
    s1 = s1 + "\""
    return s1

def file_selected(button, fs, player):
  global lastdir
  fn = fs.get_filename()
  # check accesibility first?
  if (posixpath.isdir(fn)):
    lastdir = fn
  else:
    lastdir = posixpath.split(fn)[0] + '/'
  out2mixd("(file%d %s)" % (player, escape_string(fn)))
  fs.destroy()

def on_select_clicked (button, num):
  f = GtkFileSelection("Select File for Player %d" % num)
  f.set_filename(lastdir)
  f.border_width = border_width
  ok = f.ok_button
  ok.show()
  ok.set_flags(CAN_DEFAULT)
  ok.grab_default()

  cancel = f.cancel_button
  cancel.show()
  cancel.set_flags(CAN_DEFAULT)
  cancel.connect("clicked", f.destroy)
  ok.connect("clicked", file_selected, f, num)
  f.show()

fixed = load_font('fixed')

eqp = 1

class EQ(GtkFrame):
    
  def __init__(self, pn):
    GtkFrame.__init__(self, 'EQ')
    self.set_border_width(border_width)
    v = GtkVBox(spacing=spacing)
    v.set_border_width(border_width)
    self.add(v)

    def make_cb(self, vn, pn):
      def foo (val, self=self, vn=vn, pn=pn):
        self.vs[vn] = val
        t = self.vs
        # print "(eq%d %f %f %f)" % (pn, t[0], t[1], t[2])
        out2mixd("(eq%d %f %f %f)" % (pn, t[0], t[1], t[2]))
      return foo

    l = GtkLabel('High')
    v.pack_start(l)
    cb = make_cb(self, 2, pn)
    d = GtkWidget(oolalow.dial(1, 0, 1, .01, .1, 0, cb))
    d.set_usize(50, 50)
    v.pack_start(d)
    
    l = GtkLabel('Mid')
    v.pack_start(l)
    cb = make_cb(self, 1, pn)
    d = GtkWidget(oolalow.dial(1, 0, 1, .01, .1, 0, cb))
    d.set_usize(50, 50)
    v.pack_start(d)
    
    l = GtkLabel('Low')
    v.pack_start(l)
    cb = make_cb(self, 0, pn)
    d = GtkWidget(oolalow.dial(1, 0, 1, .01, .1, 0, cb))
    d.set_usize(50, 50)
    v.pack_start(d)

    self.vs = [1.0, 1.0, 1.0]
    
  def show(self):
    self.show_all()

class PlayPosition(GtkVBox):
  def __init__(self):
    GtkVBox.__init__(self)
    self.hbox3 = GtkHBox(spacing=spacing)
    self.hbox3.show()
    self.hbox3.set_border_width(border_width)
    self.pack_start(self.hbox3)

    self.size = GtkLabel('    0:00.00')
    self.size.set_style(self.size.get_style().copy())
    self.size.get_style().font = fixed
    self.size.show()
    self.hbox3.pack_start(self.size)

    self.pos = GtkLabel('    0:00.00')
    self.pos.set_style(self.pos.get_style().copy())
    self.pos.get_style().font = fixed
    self.pos.show()
    self.hbox3.pack_start(self.pos)
    
    self.npos = GtkLabel('    0:00.00')
    self.npos.set_style(self.npos.get_style().copy())
    self.npos.get_style().font = fixed
    self.npos.show()
    self.hbox3.pack_start(self.npos)
    
    adj = GtkAdjustment(0, 0, 0, 1, 10, 0)
    self.pb = GtkHScale(adj)
    self.pb.set_draw_value(FALSE)
    self.pb.show()
    self.pb.connect('key_press_event', self.pbkeypress)
    self.pack_start(self.pb)

    self.sizef = None
    self.pos_sigid = None
    
  def set(self, p):
    self.pos.set_text(float2hms(p))

    adj = self.pb.get_adjustment()
  
    if (self.pos_sigid != None):
      adj.signal_handler_block(self.pos_sigid)
      adj.set_value(p)
    if (self.pos_sigid != None):
      adj.signal_handler_unblock(self.pos_sigid)
    adj.emit('changed')

    if (self.sizef): self.npos.set_text(float2hms(p - self.sizef))

  def new_stream(self, size, cb):
    adj = GtkAdjustment(0, 0, size, 1, 10, 0)
    def foo(adj, cb=cb): cb(adj.value)
    self.pos_sigid = adj.connect('value_changed', foo)
    self.pb.set_adjustment(adj)
    self.sizef = size
    if (size):
      self.size.set_text(float2hms(size))
    else:
      self.size.set_text('   unknown')

  def pbkeypress(self, obj, event):
    if (event.type == GDK.KEY_PRESS):
      #sys.stderr.write('press scale:' + `GDK.Left` + '  ' + `GDK.MOD1_MASK` + ' ' + `event.time` + ' ' + `event.state` + ' ' + `event.keyval` + ' ' + `event.string` + "\n")
      
      if (event.keyval == GDK.Left):
        if (event.state == (GDK.MOD1_MASK | GDK.CONTROL_MASK)):
          adj = self.pb.get_adjustment()
          adj.set_value(adj.value - .01)
          self.pb.emit_stop_by_name('key_press_event')
          return TRUE
        if (event.state == (GDK.MOD1_MASK)):
          adj = self.pb.get_adjustment()
          adj.set_value(adj.value - .1)
          self.pb.emit_stop_by_name('key_press_event')
          return TRUE
      if (event.keyval == GDK.Right):
        if(event.state == (GDK.MOD1_MASK | GDK.CONTROL_MASK)):
          adj = self.pb.get_adjustment()
          adj.set_value(adj.value + .01)
          self.pb.emit_stop_by_name('key_press_event')
          return TRUE
        if (event.state == (GDK.MOD1_MASK)):
          adj = self.pb.get_adjustment()
          adj.set_value(adj.value + .1)
          self.pb.emit_stop_by_name('key_press_event')
          return TRUE

class Player(GtkFrame):
 def __init__(self, num):

  GtkFrame.__init__(self, "Player" + ("%d" % num))
  GtkFrame.show(self)

  self.vbox = GtkVBox(spacing=spacing)
  self.vbox.set_border_width(border_width)
  self.vbox.show()
  GtkFrame.add(self, self.vbox)
  self.hbox0 = GtkHBox(spacing=spacing)
  self.hbox0.show()
  self.vbox.pack_start(self.hbox0)

  self.fname = GtkEntry()
  self.fname.set_editable(0)
  self.fname.unset_flags(CAN_FOCUS)
  self.fname.show()
  self.hbox0.pack_start(self.fname)
  self.select = GtkButton ("Select")
  self.select.show()
  self.hbox0.pack_start(self.select, expand=FALSE, fill=FALSE)
  self.select.connect("clicked", on_select_clicked, num)

  self.pp = PlayPosition()
  self.pp.show()
  self.vbox.pack_start(self.pp)

  self.hbox1 = GtkHBox(spacing=spacing)
  self.hbox1.show()
  self.hbox1.set_border_width(border_width)
  self.vbox.pack_start(self.hbox1)
    
  vboxr = GtkVBox(spacing=spacing)
  vboxr.set_border_width(border_width)
  vboxr.show()
  self.hbox1.pack_start(vboxr);

  l = GtkLabel('Rate')
  l.show()
  vboxr.pack_start(l, expand=FALSE)
  fmt = "(rate%d %%f)" % num
  self.scale2 = GtkWidget(oolalow.ovscale(1.0, 0.0, 2.0, .01, .1, 2,
                                          out2mixd, fmt,
                                          ("(rate%d 0.0)" % num),
                                          ("(rate%d %%f)" % num)))
  self.scale2.show()
  vboxr.pack_start(self.scale2)
  
  b = GtkButton('1.0')
  b.show()
  vboxr.pack_start(b, expand=FALSE)
  b.connect('clicked', lambda b, self=self: self.set_rate(1.0))
            
  vboxl = GtkVBox(spacing=spacing)
  vboxl.set_border_width(border_width)
  vboxl.show()
  self.hbox1.pack_start(vboxl);

  l = GtkLabel('Level')
  l.show()
  vboxl.pack_start(l, expand=FALSE)
  fmt = "(level%d %%f)" % num
  self.scale1 = GtkWidget(oolalow.ovscale(1, 0, 1, .01, .1, 2,
                                          out2mixd, fmt, "", ""))
  self.scale1.show()
  vboxl.pack_start(self.scale1, expand=TRUE, fill=TRUE)

  b = GtkButton('Max')
  b.show()
  vboxl.pack_start(b, expand=FALSE)
  b.connect('clicked', lambda b, self=self: self.set_level(1.0))
            
  lvb = GtkVBox(spacing=spacing)
  lvb.show()
  self.lh = LevelHold.GtkLevelHold(200, 10, 20, 20)
  self.lh.show()
  def fade(lh=self.lh):
    #sys.stderr.write("fading %s\n" % str(lh))
    lh.fade()
    return 1
  timeout_add(200, fade)
  lvb.pack_end(self.lh, expand=FALSE)
  self.hbox1.pack_start(lvb, expand=TRUE, fill=FALSE)

  if (eqp):
    self.eq = EQ(num)
    self.eq.show()
    self.hbox1.pack_start(self.eq)
  else:
    self.eq = None
  
  self.hboxn = GtkHBox(spacing=spacing)
  self.hboxn.set_border_width(border_width)
  self.hboxn.show()
  self.vbox.pack_start(self.hboxn)

  markf = GtkFrame('Mark')
  markf.show()
  self.hboxn.pack_start(markf)

  markh = GtkHBox(spacing=spacing)
  markh.show()
  markh.set_border_width(border_width)
  markf.add(markh)

  mark = GtkButton('Set')
  mark.show()
  markh.pack_start(mark, expand=FALSE)
  mark.connect('clicked', lambda b, self=self: self.set_mark())

  goto_mark = GtkButton('Goto')
  goto_mark.show()
  markh.pack_start(goto_mark, expand=FALSE)
  goto_mark.connect('clicked', lambda b, self=self: self.goto_mark())

  loopf = GtkFrame('Loop')
  loopf.show()
  self.hboxn.pack_start(loopf)

  looph = GtkHBox(spacing=spacing)
  looph.show()
  looph.set_border_width(border_width)
  loopf.add(looph)

  loop_to_mark = GtkButton('Set')
  loop_to_mark.show()
  looph.pack_start(loop_to_mark, expand=FALSE)
  loop_to_mark.connect('clicked', lambda b, self=self: self.loop_to_mark())

  clear_loop = GtkButton('Clear')
  clear_loop.show()
  looph.pack_start(clear_loop, expand=FALSE)
  clear_loop.connect('clicked', lambda b, self=self: self.clear_loop())

  vpause = GtkVBox()
  vpause.set_border_width(border_width)
  vpause.show()
  self.hboxn.pack_start(vpause)
  pause = GtkToggleButton('Pause')
  pause.show()
  vpause.pack_start(pause, expand=FALSE)
  def do_pause(b, self=self):
    if (b.active):
      self.pause()
    else:
      self.unpause()
  pause.connect('toggled', do_pause)
  self.pauseb = pause
  
  self.num = num
  self.sizef = None
  self.posf = 0.0
  self.mark = 0.0
  self.saved_rate = 1.0
  self.fine_rate = 0.0
  self.coarse_rate = 1.0

 def set_fine_rate(self, f):
   self.fine_rate = f
   self.set_rate(self.coarse_rate + self.fine_rate)
   
 def set_coarse_rate(self, f):
   self.coarse_rate = f
   self.set_rate(self.coarse_rate + self.fine_rate)
   
 def newpos(self, v):
  out2mixd("(position%d %f)\n" % (self.num, v))

 def set_mark(self, f=None):
  if (not f): f = self.posf
  self.mark = f

 def goto_mark(self):
  out2mixd("(position%d %f)" % (self.num, self.mark))

 def loop_to_mark(self):
  if (self.posf > self.mark):
   out2mixd("(setloop%d %f %f)" % (self.num, self.mark, self.posf))
  else:
   out2mixd("(setloop%d %f %f)" % (self.num, self.posf, self.mark))

 def clear_loop(self):
  out2mixd("(clearloop%d)" % self.num)
  
 def pause(self):
  if not self.pauseb.active:
    self.pauseb.set_active(TRUE)
  oolalow.ovscale_pause(self.scale2)
  return 1                              # for use in logical operations
  
 def unpause(self):
  if self.pauseb.active:
    self.pauseb.set_active(FALSE)
  oolalow.ovscale_unpause(self.scale2)
  return 1                              # for use in logical operations
  
 def set_file(self, f, size):
  self.fname.set_text(f)
  self.pp.new_stream(size, self.newpos)

 def set_pos(self, p):
   self.posf = p
   self.pp.set(p)

 def set_rate(self, r):
   #print 'in set_rate(%f)' % r
   oolalow.ovscale_set(self.scale2, r)

 def set_level(self, r):
   # print "set_level(%d, %f)" % (self.num, r)
   oolalow.ovscale_set(self.scale1, r)

 def energy(self, e):
   #sys.stderr.write("energy%d %f %f\n" % (self.num, e))
   self.lh.set_level(e)
   
def cue_toggled(button, num):
  out2mixd("(cue%d)" % num)

def do_about(*args):
  d = GtkDialog()
  d.show()
  d.vbox.set_border_width(border_width)
  ok = GtkButton('OK')
  ok.show()
  ok.set_flags(CAN_DEFAULT)
  ok.connect('clicked', d.destroy)
  d.action_area.pack_start(ok, expand=FALSE)
  l = GtkLabel("oola boola\n" \
               + "version " + version + "\n" \
               + "by est@hyperreal.org")
  l.show()
  d.vbox.pack_start(l, padding=10)
  ok.grab_default()

menuitems = [
    ('/_File', None, None, 0, '<Branch>'),
    ('/File/_Quit', '<control>Q', doquit, 0, None),

    ('/_Help', None, None, 0, '<Branch>'),
    ('/Help/_About', None, do_about, 0, None),
]

lastodir = "./"

ofile = GtkEntry()
ofile.show()
ofile.set_editable(0)
ofile.unset_flags(CAN_FOCUS)

def on_ofile_selected(b, fs):
  global lastodir
  fn = fs.get_filename()
  # check accesibility first?
  if (posixpath.isdir(fn)):
    lastdir = fn
  else:
    lastdir = posixpath.split(fn)[0] + '/'
  out2mixd("(output-to-file %s)" % escape_string(fn))
  fs.destroy()

def select_ofile(b):
  f = GtkFileSelection("Select File for Sound Output")
  f.set_filename(lastodir)
  f.border_width = border_width
  ok = f.ok_button
  ok.show()
  ok.set_flags(CAN_DEFAULT)

  cancel = f.cancel_button
  cancel.show()
  cancel.set_flags(CAN_DEFAULT)
  cancel.grab_default()
  cancel.connect("clicked", f.destroy)
  ok.connect("clicked", on_ofile_selected, f)
  f.show()

def end_ofile(b):
  out2mixd("(end-file-output)")

def munge_styles(w):
  return
  s = w.get_style()
  cm = w.get_window().colormap

  # these garish bindings are useful for figuring out what's going on :)
  # but i'd *really* like to assign a separate color to every facet
  bg = cm.alloc(0x8000, 0, 0)
  black = cm.alloc(0x8000, 0xFFFF, 0x8000)
  white = cm.alloc(0xFFFF, 0, 0)
  light = cm.alloc(0x8000, 0, 0xFFFF)
  mid = cm.alloc(0, 0xFFFF, 0)
  dark = cm.alloc(0, 0, 0xFFFF)
  text = cm.alloc(0x8000, 0, 0x8000)
  base = cm.alloc(0xFFFF, 0x8000, 0)
  fg = cm.alloc(0, 0xFFFF, 0x8000)
  
  black = cm.alloc(0, 0, 0)
  base = cm.alloc(0, 0, 0)
  dark = cm.alloc(0xFFFF, 0xFFFF, 0xFFFF)

  for m in range(0, 5):
    for facet in 'foreground', 'background':
      setattr(s.bg_gc[m], facet, bg)
  for m in range(0, 5):
    for facet in 'foreground', 'background':
      setattr(s.fg_gc[m], facet, fg)
  for m in range(0, 5):
    for facet in 'foreground', 'background':
      setattr(s.light_gc[m], facet, light)
  for m in range(0, 5):
    for facet in 'foreground', 'background':
      setattr(s.dark_gc[m], facet, dark)
  for m in range(0, 5):
    for facet in 'foreground', 'background':
      setattr(s.mid_gc[m], facet, mid)
  for m in range(0, 5):
    for facet in 'foreground', 'background':
      setattr(s.text_gc[m], facet, text)
  for m in range(0, 5):
    for facet in 'foreground', 'background':
      setattr(s.base_gc[m], facet, base)
  for facet in 'foreground', 'background':
    setattr(s.black_gc, facet, black)
  for facet in 'foreground', 'background':
      setattr(s.white_gc, facet, white)

def xfade(v):
  if (xsigid != None):
    xadj.signal_handler_block(xsigid)
  xadj.set_value(v)
  if (xsigid != None):
    xadj.signal_handler_unblock(xsigid)
  xadj.emit('changed')
  
  out2mixd("(xfade %f %f)" % (1.0 - v, v))

def xfading(adj):
  if (adj.value == 0):
    players[0].unpause()
  elif (adj.value == 1):
    players[1].unpause()
      
  out2mixd("(xfade %f %f)" % (1.0 - adj.value, adj.value))
  
def create_main_window():
  global xadj, xsigid

  win = GtkWindow()
  win.set_name("main window")
  #win.set_usize(300, 340)
  win.set_uposition(20, 20)
  win.connect("destroy", doquit)
  win.connect("delete_event", doquit)
  win.set_title("oola boola")

  vbox0 = GtkVBox()
  vbox0.show()
  ag = GtkAccelGroup()
  ifact = GtkItemFactory(GtkMenuBar, '<main>', ag)
  ifact.create_items(menuitems)
  ag.attach(win)
  mbar = ifact.get_widget('<main>')
  mbar.show()
  vbox0.pack_start(mbar)
  win.add(vbox0)
  hbox0 = GtkHBox()
  hbox0.show()
  vbox0.pack_start(hbox0)

  oframe = GtkFrame("Output File")
  oframe.set_border_width(border_width)
  oframe.show()
  hbox0.pack_start(oframe, expand=0)

  oh = GtkHBox(spacing=spacing)
  oh.set_border_width(border_width)
  oh.show()
  oframe.add(oh)

  oh.pack_start(ofile)
  select = GtkButton("Select")
  select.show()
  oh.pack_start(select, expand=FALSE, fill=FALSE)
  select.connect("clicked", select_ofile)
  end = GtkButton('End')
  end.show()
  oh.pack_start(end, expand=FALSE, fill=FALSE)
  end.connect("clicked", end_ofile)

  if cue_channel:
    cframe = GtkFrame("Cue Channel")
    cframe.set_border_width(border_width)
    cframe.show()
    hbox0.pack_start(cframe, expand=0)
    
    hboxc = GtkHBox()
    hboxc.show()
    cuebutton0 = GtkCheckButton("Player0")
    cuebutton0.show()
    cuebutton0.connect("toggled", cue_toggled, 0)
    hboxc.pack_start(cuebutton0)
    
    cuebutton1 = GtkCheckButton("Player1")
    cuebutton1.show()
    cuebutton1.connect("toggled", cue_toggled, 1)
    hboxc.pack_start(cuebutton1)
    
    cframe.add(hboxc)

  hboxp = GtkHBox(spacing=spacing)
  hboxp.set_border_width(border_width)
  hboxp.show()
  
  player0 = players[0]
  player0.show()
  hboxp.pack_start(player0)

  player1 = players[1]
  player1.show()
  hboxp.pack_start(player1)

  vbox0.pack_start(hboxp)

  xvbox = GtkVBox(spacing=spacing)
  xvbox.set_border_width(border_width)
  xvbox.show()
  vbox0.pack_start(xvbox, expand=FALSE)
  l = GtkLabel("XFade")
  l.show()
  xvbox.pack_start(l, expand=FALSE)

  xhbox = GtkHBox(spacing=spacing)
  xhbox.set_border_width(border_width)
  xhbox.show()
  xvbox.pack_start(xhbox, expand=FALSE)
  
  xadj = GtkAdjustment(.5, 0, 1, .01, .1, 0)
  xsigid = xadj.connect("value_changed", xfading)
  xfade = GtkHScale(xadj)
  xfade.set_draw_value(FALSE)
  xfade.show()
  xhbox.pack_start(xfade)
  xcenter = GtkButton("Center")
  xcenter.show()
  xhbox.pack_start(xcenter, expand=FALSE)
  def do_xcenter(b, adj=xadj):
    adj.set_value(0.5)
  xcenter.connect('clicked', do_xcenter)
  
  win.connect('key_press_event', keypress)
  win.connect('key_release_event', keypress)

  win.show()

  munge_styles(win)

def keypress(win, event):
  if (event.type == GDK.KEY_PRESS):
    if (event.keyval == 101 
        and event.state == (GDK.CONTROL_MASK | GDK.MOD1_MASK)):
      dorepl()
  return
  if (event.type == GDK.KEY_RELEASE):
    print 'release', event.time, event.state, event.keyval, event.string
  if (event.type == GDK.KEY_PRESS):
    print 'press', event.time, event.state, event.keyval, event.string

last_scratch = {}
last_scratch_time = {}
scratch_factor = 2
# figger out what units this is in!
scratch_check_rate = 10.0

def scratch(player, pos):
  t = time.time()
  last = last_scratch.get(player)
  if (last):
    dt = t - last_scratch_time[player]
    ds = pos - last_scratch[player]
    last_scratch[player] = pos
    last_scratch_time[player] = t
    rate = (ds / dt) * scratch_factor
    sys.stderr.write("player %d scratch rate %f\n" % (player, rate))
    sys.stderr.flush()
    players[player].set_rate(rate, 1)
    def scratch_check(player=player, t0=t):
      if (last_scratch_time[player] == t0):
        players[player].set_rate(0, 1)
        sys.stderr.write("player %d scratchout\n" % player)
        sys.stderr.flush()
      return 0
    msecs = 200 #1000 * int(round(scratch_check_rate / rate))
    #sys.stderr.write(`msecs` + "\n")
    #sys.stderr.flush()
    timeout_add(msecs, scratch_check)
  else:
    last_scratch[player] = pos
    last_scratch_time[player] = t

def isregular(f):
  try: s = os.stat(f)
  except: return 0
  return stat.S_ISREG(s[0])


orig_stdout = sys.stdout

def dorepl():
 s = sys.stdout
 sys.stdout = orig_stdout

 while(1):
  c = None
  p = '>>> '
  i = ''

  while (not c):
    try: l = raw_input(p) + "\n"
    except EOFError:
      print
      print 'later...'
      sys.stdout = s
      return
    i = i + l
    try: c = code.compile_command(i)
    except SyntaxError, v:
      print 'Syntax error:', v
      break
    if (c):
      try: exec(c)
      except Exception, v: print 'runtime exception occured:', sys.exc_info()
      break
    else: p = '... '


mi = MidiIn2.MidiIn2()

# default midi bindings (belong in separate file)
mi.cb['control-change'][5][1] = lambda v: players[0].set_coarse_rate(v / (127.0 / 2))
mi.cb['control-change'][5][2] = lambda v: players[0].set_fine_rate(v / (5 * 127.0))
mi.cb['control-change'][5][3] = lambda v: players[0].set_level(v / 127.0)

mi.cb['control-change'][5][8] = lambda v: players[1].set_coarse_rate(v / (127.0 / 2))
mi.cb['control-change'][5][9] = lambda v: players[1].set_fine_rate(v / (5 * 127.0))
mi.cb['control-change'][5][10] = lambda v: players[1].set_level(v / 127.0)

mi.cb['control-change'][5][7] = lambda v: xfade(v / 127.0)

#mi.cb['control-change'][5][4] = lambda v: scratch(0, v/127.0)
#mi.cb['control-change'][5][11] = lambda v: scratch(1, v/127.0)

mi.cb['note-on'][1][60] = lambda v: \
                          (v == 0) \
                          and players[0].unpause() \
                          or players[0].pause()
mi.cb['note-off'][1][60] = lambda v: players[0].unpause()
  
mi.cb['note-on'][1][62] = lambda v: \
                          (v == 0) \
                          and players[1].unpause() \
                          or players[1].pause()
mi.cb['note-off'][1][62] = lambda v: players[1].unpause()
  

music_directory = None
mixd_options = []
midi_device = '/dev/midi'
fragment = None

home = os.environ.get('HOME', None)

if (home and isregular(home + '/.oola/rc')):
  try: execfile(home + '/.oola/rc')
  except: print "problem executing `%s'" % home + '/.oola/rc'

oola = os.environ.get('OOLAHOME', '.')

if not music_directory:
    music_directory = os.environ.get('OOLA_MUSIC_DIRECTORY', './')

cue_channel = os.environ.get('OOLA_CUE_CHANNEL', cue_channel)
fragment = os.environ.get('OOLA_FRAGMENT', fragment)

mixd = os.environ.get('OOLA_MIXD', oola + '/mixd')

opts, args = getopt.getopt(sys.argv[1:],
                           "",
                           ['profile', 'music-directory=', 'exec=',
                            'cue-channel=', 'fragment=', 'version',
                            'no-eq', 'mixd=', 'midi-device='
                            'help', 'no-log'])

for opt, val in opts:
  if opt == '--profile': profilep = 1
  elif opt == '--no-log': nologp = 1
  elif opt == '--music-directory': music_directory = val
  elif opt == '--midi-device': midi_device = val
  elif opt == '--exec': exec val
  elif opt == '--cue-channel':
    cue_channel = val
  elif opt == '--fragment':
    fragment = val
  elif opt == '--mixd':
    mixd_options = mixd_options + [val]
  elif opt == '--no-eq':
    eqp = 0
  elif opt == '--help':
    print '''
oolaboola - dj software - version %s

Usage: oolaboola [options]

Options:
 --help                       Print this message and exit.
 --version                    Print the version number of oolaboola and exit.
 --music-directory DIRECTORY  Set the default directory for opening soud
                              files to DIRECTORY.
 --midi-device DEVICE         Get MIDI input from DEVICE.
 --exec PYTHON-SUITE          Execute the python suite PYTHON-SUITE after
                              reading any rc file.
 --cue-channel DEVICE         Output cue data to DEVICE.
 --fragment FRAGMENT-SPEC     Set output OSS fragment to FRAGMENT.
                              FRAGMENT should be an 8 character hex string
 --profile                    Turn on python profiling of the GUI.

 ''' % version
    sys.exit(0)
    
  elif opt == '--version':
    print '''oolaboola version %s, by Eric S. Tiedemann
Copyright (C) 1999 Eric S. Tiedemann
This is free software; see the source for copying conditions.
There is NO warranty; not even for MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE.

Report bugs to <est@hyperreal.org>.
''' % version
    sys.exit(0)

if fragment:
  mixd_options = mixd_options + ['--fragment', fragment]
  
if cue_channel:
  mixd_options = mixd_options + ['--cue-channel', cue_channel]
  
if eqp:
  mixd_options = mixd_options + ['--eq']

oolalogdir = None

if not oolalogdir and home:
  oolalogdir = home + '/oola-logs'

def start_logging():
  global oolalog
  if not nologp:
    if not oolalogdir:
      print "No logging directory defined!"
      sys.exit(1)
    try:
      mode = os.stat(oolalogdir)[stat.ST_MODE]
    except os.error, e:
      print "couldn't access `%s': %s" % (oolalogdir, e[1])
      print "continuing without logging"
      return
    if not stat.S_ISDIR(mode):
      print "logging directory `%s' isn't a directory!" \
            % oolalogdir
      print "continuing without logging"
    else:
      oolalogn = oolalogdir + '/' + 'oola' + \
                 time.strftime('-%Y-%m-%d-%H:%M:%S',
                               time.localtime(time.time()))
      try:
        oolalog = open(oolalogn, 'w')
      except IOError, e:
        print "Couldn't open `%s' for logging: %s" % (oolalogn, e[1])
        print "continuing without logging"

start_logging()

lastdir = music_directory

if (lastdir[-1] != '/'): lastdir = lastdir + '/'

def handle_midi(fd, huh):
  bs = os.read(fd, 1000)
  #print len(bs)
  #print "in handle_midi(%d)" % len(bs)
  mi.inbytes(bs)

try:
  midi = os.open(midi_device, os.O_RDONLY)
except os.error, e:
  sys.stderr.write("Couldn't open %s for input: %s\n" % (midi_device, e[1]))
  midi = None

if (not isregular(mixd)):
  print "Couldn't find mixd in", mixd
  sys.exit(1)

r0, w0 = os.pipe()
r1, w1 = os.pipe()

mixdpid = os.fork()

energy0 = sexpr.intern('energy0')
position0 = sexpr.intern('position0')
file0 = sexpr.intern('file0')
energy1 = sexpr.intern('energy1')
position1 = sexpr.intern('position1')
file1 = sexpr.intern('file1')
output_to_file = sexpr.intern('output-to-file')
append_output_to_file = sexpr.intern('append-output-to-file')
end_file_output = sexpr.intern('end-file-output')

def handle_mixdin(fd, huh):
  s = sexpr.read(mixdinf)
  #sys.stderr.write(`s` + "\n")
  if (s.pairp()):
    c = s.car

    if not s.cdr.pairp(): t = ()
    elif not s.cdr.cdr.pairp(): t = (s.cadr,)
    else: t = (s.cadr, s.caddr)

    try: apply(dispatch[c], t)
    except KeyError: pass

def query_positions():
  out2mixd('(position0-get)', 1)
  out2mixd('(position1-get)', 1)
  out2mixd('(energy0-get)', 1)
  out2mixd('(energy1-get)', 1)
  return 1

if (mixdpid == 0):
  os.close(0)
  os.close(1)
  os.dup(r0)
  os.dup(w1)
  os.close(r0)
  os.close(w0)
  os.close(r1)
  os.close(w1)
  os.execv(mixd, ['mixd'] + mixd_options)
else:
  # start a super-real-time killer process to void reboot on uniprocessors
  # os.system("/home/est/ooladev/src/killit 600000000 %d &" % mixdpid)
  os.close(r0)
  os.close(w1)
  mixdout = os.fdopen(w0, 'w')
  mixdin = r1
  mixdinf = os.fdopen(mixdin, 'r')
  players = (Player(0), Player(1))
  dispatch = {
    energy0:players[0].energy,
    position0:players[0].set_pos,
    file0:players[0].set_file,
    
    energy1:players[1].energy,
    position1:players[1].set_pos,
    file1:players[1].set_file,
    
    output_to_file:ofile.set_text,
    append_output_to_file:ofile.set_text,
    end_file_output:ofile.set_text,
    }
  
  create_main_window()
  input_add(mixdin, GDK.INPUT_READ, handle_mixdin)
  if midi:
    fcntl.fcntl(midi, FCNTL.F_SETFL,
                (os.O_NONBLOCK | fcntl.fcntl(midi, FCNTL.F_GETFL)))
    input_add(midi, GDK.INPUT_READ, handle_midi)
  timeout_add(50, query_positions)
  if (oola): out2mixd("(file0 %s)" % escape_string(oola + '/oola.wav'))

  if (profilep):
    import profile
    p = profile.run('mainloop()', 'oolastats')
    import pstats
    p = pstats.Stats('oolastats')
    p.sort_stats('cumulative').print_stats(20)
    p.sort_stats('time').print_stats(20)
  else:
    #os.wait()
    mainloop()
    sys.exit(0)
